<?php namespace TierPricingTable\Frontend;

use TierPricingTable\Settings\Settings;
use WC_Product;
use TierPricingTable\PriceManager;

/**
 * Class CatalogPriceManager
 *
 * @package TierPricingTable
 */
class CatalogPriceManager {

	/**
	 * @var Settings
	 */
	private $settings;

	/**
	 * CatalogPriceManager constructor.
	 *
	 * @param Settings $settings
	 */
	public function __construct( Settings $settings ) {
		$this->settings = $settings;

		if ( $this->isEnable() && ! is_admin() ) {
			add_filter( 'woocommerce_get_price_html', [ $this, 'formatPrice' ], 999, 2 );
		}
	}

	/**
	 * Change logic showing prince at catalog for product with tiered price rules
	 *
	 * @param string $priceHtml
	 * @param WC_Product $product
	 *
	 * @return string
	 */
	public function formatPrice( $priceHtml, $product ) {
		if ( ! is_product() ) {
			if ( in_array( $product->get_type(), [ 'simple', 'variation' ] ) ) {
				$rules = PriceManager::getPriceRules( $product->get_id() );

				if ( ! empty( $rules ) ) {
					$displayPriceType = $this->getDisplayType();

					if ( $displayPriceType === 'range' ) {
						return $this->getRange( $rules, $product );
					} else {
						return $this->getLowestPrice( $rules, $product );
					}
				}
			}
		}

		return $priceHtml;
	}

	/**
	 * Get range from lowest to highest price from price rules
	 *
	 * @param array $rules
	 * @param WC_Product $product
	 *
	 * @return string
	 */
	protected function getRange( $rules, $product ) {
		if ( PriceManager::getPricingType( $product->get_id() ) === 'percentage' ) {

			$lowest  = wc_price( PriceManager::getPriceByPercentDiscount( $product->get_price(),
				array_pop( $rules ) ) );

			$highest = wc_price( $product->get_price() );

			return $lowest . ' - ' . $highest;
		}

		return wc_price( array_pop( $rules ) ) . ' - ' . wc_price( $product->get_price() );
	}

	/**
	 * Get lowest price from price rules
	 *
	 * @param array $rules
	 * @param WC_Product $product
	 *
	 * @return string
	 */
	protected function getLowestPrice( $rules, $product ) {
		if ( PriceManager::getPricingType( $product->get_id() ) === 'percentage' ) {
			$lowest = wc_price( PriceManager::getPriceByPercentDiscount( $product->get_price(),
				array_shift( $rules ) ) );
		} else {
			$lowest = wc_price( array_pop( $rules ) );
		}

		return $this->getLowestPrefix() . ' ' . $lowest;
	}

	public function getLowestPrefix() {
		return $this->settings->get( 'lowest_prefix', __( 'From', 'tier-pricing-table' ) );
	}

	public function isEnable() {
		return $this->settings->get( 'tiered_price_at_catalog', 'yes' ) === 'yes';
	}

	public function getDisplayType() {
		return $this->settings->get( 'tiered_price_at_catalog_type', 'range' );
	}

}